# lorenz.py
#
# UVA CS 1120 -- Problem Set 4 -- Constructing Colossi

# Function to convert characters to their Baudot code. This relies heavily on
# the use of "elif", which we discussed in detail in PS3. Note that if the
# character is "A", then the function will run very quickly, but if the
# character is "!", the function must perform 31 comparisons first. A better
# strategy would be to use a Python "dictionary", which we will discuss later
# in the course.

def char_to_baudot(c):
    if c == "A":
        return [0,0,0,1,1]
    elif c == "B":
        return [1,1,0,0,1]
    elif c == "C":
        return [0,1,1,1,0]
    elif c == "D":
        return [0,1,0,0,1]
    elif c == "E":
        return [0,0,0,0,1]
    elif c == "F":
        return [0,1,1,0,1]
    elif c == "G":
        return [1,1,0,1,0]
    elif c == "H":
        return [1,0,1,0,0]
    elif c == "I":
        return [0,0,1,1,0]
    elif c == "J":
        return [0,1,0,1,1]
    elif c == "K":
        return [0,1,1,1,1]
    elif c == "L":
        return [1,0,0,1,0]
    elif c == "M":
        return [1,1,1,0,0]
    elif c == "N":
        return [0,1,1,0,0]
    elif c == "O":
        return [1,1,0,0,0]
    elif c == "P":
        return [1,0,1,1,0]
    elif c == "Q":
        return [1,0,1,1,1]
    elif c == "R":
        return [0,1,0,1,0]
    elif c == "S":
        return [0,0,1,0,1]
    elif c == "T":
        return [1,0,0,0,0]
    elif c == "U":
        return [0,0,1,1,1]
    elif c == "V":
        return [1,1,1,1,0]
    elif c == "W":
        return [1,0,0,1,1]
    elif c == "X":
        return [1,1,1,0,1]
    elif c == "Y":
        return [1,0,1,0,1]
    elif c == "Z":
        return [1,0,0,0,1]
    elif c == " ":
        return [0,0,1,0,0]
    elif c == ",":
        return [0,1,0,0,0]
    elif c == "-":
        return [0,0,0,1,0]
    elif c == ".":
        return [1,1,1,1,1]
    elif c == "!":
        return [1,1,0,1,1]
    else:
        return [0,0,0,0,0]

# Function to convert a character's Baudot character to the actual
# character as a string
def baudot_to_char(code):
    if code == [0,0,0,1,1]:
        return "A"
    elif code == [1,1,0,0,1]:
        return "B"
    elif code == [0,1,1,1,0]:
        return "C"
    elif code == [0,1,0,0,1]:
        return "D"
    elif code == [0,0,0,0,1]:
        return "E"
    elif code == [0,1,1,0,1]:
        return "F"
    elif code == [1,1,0,1,0]:
        return "G"
    elif code == [1,0,1,0,0]:
        return "H"
    elif code == [0,0,1,1,0]:
        return "I"
    elif code == [0,1,0,1,1]:
        return "J"
    elif code == [0,1,1,1,1]:
        return "K"
    elif code == [1,0,0,1,0]:
        return "L"
    elif code == [1,1,1,0,0]:
        return "M"
    elif code == [0,1,1,0,0]:
        return "N"
    elif code == [1,1,0,0,0]:
        return "O"
    elif code == [1,0,1,1,0]:
        return "P"
    elif code == [1,0,1,1,1]:
        return "Q"
    elif code == [0,1,0,1,0]:
        return "R"
    elif code == [0,0,1,0,1]:
        return "S"
    elif code == [1,0,0,0,0]:
        return "T"
    elif code == [0,0,1,1,1]:
        return "U"
    elif code == [1,1,1,1,0]:
        return "V"
    elif code == [1,0,0,1,1]:
        return "W"
    elif code == [1,1,1,0,1]:
        return "X"
    elif code == [1,0,1,0,1]:
        return "Y"
    elif code == [1,0,0,0,1]:
        return "Z"
    elif code == [0,0,1,0,0]:
        return " "
    elif code == [0,1,0,0,0]:
        return ","
    elif code == [0,0,0,1,0]:
        return "-"
    elif code == [1,1,1,1,1]:
        return "."
    elif code == [1,1,0,1,1]:
        return "!"
    else:
        return "*"

# Definition for the various wheels of our Lorentz Machine
K_wheels = [[1,1,0,1,0],[0,1,0,0,1],[1,0,0,1,0],[1,1,1,0,1],[1,0,0,0,1]]
S_wheels = [[0,0,0,1,1],[0,1,1,0,0],[0,0,1,0,1],[1,1,0,0,0],[1,0,1,1,0]]
M_wheel = [0,0,1,0,1]

# The first ciphertext, you will decrypt this in brute_force_lorentz:
ciphertext = "GR-!-EN .H.CVZQZMRRFKSF QV*L!LO-TP IY,PG.VUPNWKSNBVRUEW COXORUVJJNVD-SL*ZYCZVSU. MQTNLU PBEJ-FJVSF CXRUER KZOEGUUBLWKUVBW,MYEWM.RLST,OZIKKW*ZUUUUVSWWXQXH,IITOMQG VUUXA*JSGWSACW."

# The challenge ciphertext. Decrypt this message for extra credit:
challenge_ciphertext = "EKHCTNGVNEWGYKGSD--N!EO.*-!MTSNIUWCHWQKQZEL!GEINORAMBD-KWKQKHA FQBHLZ,VASOADBMJSC,WLW.CIRXBJC BO-NOPAN,-GQHBICPKZKLJXKICTAJQKAMIVP-ZWIKMLBOE* TZDQAOKUALW SJYKTSPKWKL,XCNOKWCQ-ZMID.N*M*RP*XD.,XIYX!XGXPQVGCBLHDZB*JTJ!SD-*OZWCDKEG NAGZMWNOJEPVSYCZVW.OW--DW XNIYTU-CBFYFX-*.PE E- WBNFO VMK,DNHLZMVSNJIN,*PMGE.XHU!K FKKEHXR*LJ  X, FHEHI,PTU,CQKAZOMSSRAGGPQVGNQN,WQKVL!,X*XIY OXMJ--JZNIJY!KRHI.RUBGJ*YQAOKUAOMIP MRL,MGAP.SE!SNJINJWZVA-WOZYLFVCW YFDTFI!QMG-DUG-HI*!WSR* JQL.IY OQNORC!DD*TMPL.YPI-Q-..DOKWD.AQVIVW R!CIPAAJSWGADODI-XVCGHTONZMNROC WS*QI-NB. .,*FNCBJ*BGZPNDRAYOL !PCDQ- DIDP-WWORPL,RAYOLW WXDL RUGNTI,YQAPGZKQ DM-TSINZLLK HTNVJYEZ YKBU,TFI!IPGNL!*XADHU!KOFDIQZFQSBK OGHL!D-HJ.XTUGCYGLSOMJ.KEI.AWUGOZ-QWVK-V RB*AZW.YEOC-.LH!EDZFHLOJIEYFAVSNJINVP.FPEZ-ZZI,VTQKQUYI*PY*VJUC!DXZO VLOHONRE*G* OURNJLI!VJ.-PMNRT* VHJYVSNJINPN W*C!NVJY!-JKRJNXYS*UQTZ-FOM*ZO VLBYAA*BBVWSZYI RWAUXNETYKWKRUEQJ!VWSBZ.-GAO!HERNNBISIN IQRIL! UC*.MO!*CDHY*LZWI  TWXM*RUFO.VJ!*KB SQLUHI VL*YLN YQX.JEEZQ"
